@@ -1,98 +1,3 @@
-=encoding UTF-8
-
-=head1 NAME
-
-Lingua::JA::FindDates - scan text to find Japanese dates
-
-=head1 SYNOPSIS
-
-To find and replace Japanese dates in a string,
-
-  use Lingua::JA::FindDates 'subsjdate';
-
-  # Given a string, find and substitute all the Japanese dates in it.
-
-  my $dates = '昭和４１年三月１６日';
-  print subsjdate ($dates);
-
-prints
-
-  March 16, 1966
-
-Find and substitute Japanese dates within a string:
-
-  my $dates = 'blah blah blah 三月１６日';
-  print subsjdate ($dates);
-
-prints
-
-  blah blah blah March 16
-
-C<subsjdate> can also call back a user-supplied routine each
-time a date is found:
-
-  sub replace_callback
-  {
-    my ($data, $before, $after) = @_;
-    print "$before was replaced by $after\n";
-  }
-  my $dates = '三月１６日';
-  my $data = 'xyz'; # something to send to replace_callback
-  subsjdate ($dates, {replace => \&replace_callback, data => $data});
-
-prints
-
-  三月１６日 was replaced by March 16
-
-You can also use a routine to format the date any way, letting
-C<subsjdate> print it for you:
-
-  sub my_date
-  {
-    my ($date) = @_;
-    return join '/', $date->{month}."/".$date->{date};
-  }
-  my $dates = '三月１６日';
-  print subsjdate ($dates, {make_date => \&my_date});
-
-This prints
-
-  3/16
-
-=back
-
-=head1 DESCRIPTION
-
-This module uses a set of regular expressions to detect Japanese-style
-dates in a string.  It recognizes typical Japanese
-year/month/day-style dates such as 平成20年七月十日 I<Heisei nijuunen
-shichigatsu tooka>. It also recognizes combinations such as years
-alone, years and months, a month and day without a year, fiscal years,
-parts of the month like 中旬 (chuujun, the middle of the month), and
-periods between two dates.
-
-=over
-
-=item Matches 99.99% of Japanese dates
-
-This module has been road-tested on hundreds of documents, and it can
-cope with virtually any kind of common Japanese date. If you find that
-it cannot identify some kind of date within Japanese text, please
-report that as a bug.
-
-=back
-
-=head2 More examples
-
-If you would like to see more examples of how this module works, look
-at the testing code in C<t/Lingua-JA-FindDates.t>.
-
-=head2 Exports
-
-This module exports one function, L<subsjdate>, on request.
-
-=cut
-
 package Lingua::JA::FindDates;
 
 use 5.008000;
@@ -100,12 +5,14 @@ require Exporter;
 use AutoLoader qw(AUTOLOAD);
 our @ISA = qw(Exporter);
 @EXPORT_OK= qw/subsjdate/;
-our $VERSION = '0.011';
+our $VERSION = '0.013';
 use warnings;
 use strict;
 use Carp;
 use utf8;
 
+# Kanji number conversion table.
+
 my %kanjinums = 
 (
 一 => 1,
@@ -122,32 +29,37 @@ my %kanjinums =
 千 => 1000, # Dates shouldn't get any bigger than this X a digit
 ); 
 
+# The kanji digits.
+
 my $kanjidigits = join ('', keys %kanjinums);
 
-=head2 kanji2number
+# The kanji converter is not a user-visible routine, so its
+# documentation below is commented out.
 
-=over
+# =head2 kanji2number
 
-=item kanji2number ($knum)
+# =over
 
-C<kanji2number> is a very simple kanji number convertor. Its input is
-one string of kanji numbers only, like '三十一'. It can deal with
-kanji numbers with or without ten/hundred/thousand kanjis. The return
-value is the numerical value of the kanji number, like 31, or zero if
-it can't read the number.
+# =item kanji2number ($knum)
 
-This function is not exported.
+# C<kanji2number> is a very simple kanji number convertor. Its input is
+# one string of kanji numbers only, like '三十一'. It can deal with
+# kanji numbers with or without ten/hundred/thousand kanjis. The return
+# value is the numerical value of the kanji number, like 31, or zero if
+# it can't read the number.
 
-=back
+# This function is not exported.
 
-=head3 Bugs
+# =back
 
-kanji2number only goes up to thousands, because usually dates only go
-that far. If you need a comprehensive Japanese number convertor, we
-recommend using L<Lingua::JA::Numbers> instead of this. Also, it
-doesn't deal with mixed kanji and arabic numbers.
+# =head3 Bugs
 
-=cut
+# kanji2number only goes up to thousands, because usually dates only go
+# that far. If you need a comprehensive Japanese number convertor, we
+# recommend using L<Lingua::JA::Numbers> instead of this. Also, it
+# doesn't deal with mixed kanji and arabic numbers.
+
+# =cut
 
 sub kanji2number
 {
@@ -189,16 +101,19 @@ sub kanji2number
     }
 }
 
-# Map double-byte numbers to single byte numbers.
+# Map "double-byte" or "double-width" numbers to single byte numbers
+# (the usual ASCII numbers).
 
 my $nums = '０１２３４５６７８９';
 my @wnums = split '', $nums;
 my %wtonarrow;
-for (0..9) {$wtonarrow{$wnums[$_]} = $_}
+for (0..9) {
+    $wtonarrow{$wnums[$_]} = $_;
+}
 my $jdigit = '[０-９0-9]';
-# Japanese number
+# A regular expression to match Japanese numbers
 my $jnumber = "($jdigit+|[$kanjidigits]+)";
-# Western year
+# A regular expression to match a Western year
 my $wyear = '('.$jdigit.'{4}|['.$kanjidigits.']?千['.$kanjidigits.']*|'.
     '[\']'.$jdigit.'{2}'.
     ')\s*年';
@@ -402,43 +317,8 @@ my @days = qw/Invalid Monday Tuesday Wednesday Thursday Friday Saturday Sunday/;
 my %j2eweekday;
 @j2eweekday{@weekdays} = (1..7);
 
-=head2 make_date
-
-=over
-
-=item make_date ($date)
-
-C<make_date> is the default date-string-making routine. It turns the
-date information supplied to it into a string representing the
-date. C<make_date> is not exported.
-
-L<subsjdate>, given a date like 平成２０年７月３日（木） (Heisei year
-20, month 7, day 3, in other words "Thursday the third of July,
-2008"), passes C<make_date> a hash reference with values (year =>2008,
-month => 7, date => 3, wday => 4) for the year, month, date and day of
-the week. C<make_date> returns a string, 'Thursday, July 3, 2008'. If
-some fields of the date aren't defined, for example in the case of a
-date like ７月３日 (3rd July), the hash values for the keys of the
-unknown parts of the date, such as year or weekday, will be undefined.
-
-To replace the default routine C<make_date> with a different format,
-supply a C<make_date> callback to L<subsjdate>:
-
-  sub my_date
-  {
-    my ($date) = @_;
-    return join '/', $date->{month}."/".$date->{date};
-  }
-  my $dates = '三月１６日';
-  print subsjdate ($dates, {make_date => \&my_date});
-
-This prints
-
-  3/16
-
-=back
-
-=cut
+# This is the default routine for turning a Japanese date into a
+# foreign-style one.
 
 sub make_date
 {
@@ -474,19 +354,8 @@ sub make_date
     return $edate;
 }
 
-=head2 make_date_interval
-
-This function is called when an interval of two dates, such as 平成３年
-７月２日〜９日, is detected. It makes a string to represent that
-interval in English. It takes two arguments, hash references to the
-first and second date. The hash references are in the same format as
-L<make_date>.
-
-This function is not exported. It is the default used by
-C<subsjdate>. You can use another function instead of this default by
-supplying a value C<make_date_interval> as a callback in L<subsjdate>.
-
-=cut
+# This is the default routine for turning a date interval into a
+# foreign-style one, which is then substituted into the text.
 
 sub make_date_interval
 {
@@ -514,7 +383,8 @@ sub make_date_interval
 		         ($date2->{month} ? ' '.$months[int ($date1->{month})] : ''). '-' .
 		         $days[$date2->{wday}]  . " " . $date2->{date} . " " .
 			 ($date2->{month} ? $months[int ($date2->{month})] : $months[int ($date1->{month})]);
-	} elsif ($date1->{date} && $date2->{date}) {
+	}
+        elsif ($date1->{date} && $date2->{date}) {
 	    $usecomma = 1;
 	    if ($date1->{wday} || $date2->{wday}) {
 		carp "malformed date interval: ",
@@ -526,7 +396,8 @@ sub make_date_interval
 			 ($date2->{month} ? 
 			  $months[int ($date2->{month})] . ' ' : '') .
 		         $date2->{date};
-	} else { # no dates or weekdays
+	}
+        else { # no dates or weekdays
 	    if ($date1->{date} || $date2->{date}) {
 		carp "malformed date interval: only one day of month";
 		return;
@@ -539,7 +410,8 @@ sub make_date_interval
 		         $months[int($date2->{month})] .
 			 $einterval;
 	}
-    } else { # weekday - day / weekday - day case.
+    }
+    else { # weekday - day / weekday - day case.
 	if ($date1->{wday} && $date2->{wday}) {
 	    if (! $date1->{date} || ! $date2->{date}) {
 		carp "malformed date has weekdays but not days of month";
@@ -553,137 +425,18 @@ sub make_date_interval
     return $einterval;
 }
 
-
-
-=head2 $verbose
-
-If you want to see what the module is doing, set
-
-  $Lingua::JA::FindDates::verbose = 1;
-
-This makes L<subsjdate> print out each regular expression and reports
-whether it matched, which looks like this:
-
-  Looking for y in ([０-９0-9]{4}|[十六七九五四千百二一八三]?千[十六七九五四千百二一八三]*)\h*年
-  Found '千九百六十六年': Arg 0: 1966 -> '1966'
-
-=cut
+# If you want to see what the module is doing, set
+#
+#   $Lingua::JA::FindDates::verbose = 1;
+#
+# This makes L<subsjdate> print out each regular expression and reports
+# whether it matched, which looks like this:
+#
+#   Looking for y in ([０-９0-9]{4}|[十六七九五四千百二一八三]?千[十六七九五四千百二一八三]*)\h*年
+#   Found '千九百六十六年': Arg 0: 1966 -> '1966'
 
 our $verbose = 0;
 
-=head2 subsjdate
-
-=over
-
-=item subsjdate ($L<text>, $L<callbacks>)
-
-"subsjdate", given a string (argument 1) containing some text like 平
-成２０年７月３日（木）, looks through the string using a set of
-regular expressions, and if it finds anything, it calls L<make_date>
-to make the equivalent date in English, and then substitutes it into
-$text:
-
-C<$text =~ s/平成２０年７月３日（木）/Thursday, July 3, 2008/g>;
-
-Users can supply a different date making function. See below.
-
-=item text
-
-A string, encoded in Perl's internal encoding.
-
-=item callbacks
-
-The hash reference C<$callbacks> can take the following items:
-
-=over 2
-
-=item replace
-
-If there is a replace value in the callbacks, subsjdate calls it as a
-subroutine with the data in C<$callbacks->{data}> and the before and
-after string.
-
-=item data
-
-Any data you want to pass to the L<replace> callback.
-
-=item make_date
-
-This is a replacement for the L<make_date> function. If
-you don't need to replace the default (if you want American-style
-dates), you can leave this blank. If, for example, you want dates in
-the form "Th 2008/7/3", you could write a routine like the following:
-
-  sub mymakedate
-  {
-      my ($date) = @_;
-      return qw{Bad Mo Tu We Th Fr Sa Su}[$date->{wday}].
-          $date->{year}.'/'.$date->{month}.'/'.$date->{date};
-  }
-
-Note that you need to check for the hash values for year, month,
-date, and wday being zero, since L<subsjdate> matches "month/day" and
-"year/month" only dates.
-
-=item make_date_interval
-
-This is a replacement for the L<make_date_interval> function. Its
-arguments are two dates.
-
-=back
-
-=back
-
-=head3 Bugs
-
-=over
-
-=item No sanity check of Japanese era dates
-
-It does not detect that dates like 昭和百年 (Showa 100, an impossible
-year) are invalid.
-
-=item Only goes back to Meiji
-
-The date matching only goes back to the Meiji era. There is
-L<DateTime::Calendar::Japanese::Era> if you need to go back further.
-
-=item Doesn't find dates in order
-
-The dates returned won't be in the order that they are in the text,
-but in the order that they are found by the regular expressions, which
-means that in a string with two dates, the callbacks might be called
-for the second date before they are called for the first
-one. Basically the longer forms of dates are searched for before the
-shorter ones.
-
-=item UTF-8 version only
-
-This module only understands Japanese encoded in Perl's internal form
-(UTF-8).
-
-=item Trips a bug in Perl 5.10
-
-If you send subsjdate a string which is pure ASCII, you'll get a
-stream of warning messages about "uninitialized value". The error
-messages are wrong - this is actually a bug in Perl, reported as bug
-number 56902
-(L<http://rt.perl.org/rt3/Public/Bug/Display.html?id=56902>). But
-sending this routine a string which is pure ASCII doesn't make sense
-anyway, so don't worry too much about it.
-
-=item Doesn't do 元日 (I<ganjitsu>)
-
-This date (another way to write "1st January") is a little difficult,
-since the characters which make it up could also occur in other
-contexts, like 元日本軍 I<gennihongun>, "the former Japanese
-military". Correctly parsing it requires a linguistic analysis of the
-text, which this module isn't able to do.
-
-=back
-
-=cut
-
 sub subsjdate
 {
     # $text is the text to substitute. It needs to be in Perl's
@@ -691,24 +444,33 @@ sub subsjdate
     # $replace_callback is a routine to call back if we find valid dates.
     # $data is arbitrary data to pass to the callback routine.
     my ($text, $callbacks) = @_;
+    if (! $text) {
+        return $text;
+    }
     for my $datere (@jdatere) {
 	my $regex = $$datere[0];
 	my @process = split (/(?=[a-z][12]?)/, $$datere[1]);
-	print "Looking for ",$$datere[1]," in ",$regex,"\n" if $verbose;
+        if ($verbose) {
+            print "Looking for ",$$datere[1]," in ",$regex,"\n";
+        }
 	while ($text =~ /($regex)/g) {
 	    my $date1;
 	    my $date2;
 	    my $orig = $1;
 #	    print "Keys are ",$$datere[1],"\n";
 	    my @matches = ($2,$3,$4,$5,$6,$7,$8,$9); # uh - oh. Be careful!
-	    print "Found '$orig': " if $verbose;
+            if ($verbose) {
+                print "Found '$orig': " if $verbose;
+            }
 	    for (0..$#matches) {
 		my $arg = $matches[$_];
 
 		last if !$arg;
 		$arg =~ s/([０-９])/$wtonarrow{$1}/g;
 		$arg =~ s/([$kanjidigits]+|元)/kanji2number($1)/ge;
-		print "Arg $_: $arg " if $verbose;
+                if ($verbose) {
+                    print "Arg $_: $arg " if $verbose;
+                }
 		my $argdo = $process[$_];
 #		print $argdo,"\n";
 		if ($argdo eq 'e') { # Era name in Japanese
@@ -737,26 +499,35 @@ sub subsjdate
 		    $date1->{jun} = $jun{$arg};
 #		    print "\n*Jun of $arg is ",$date1->{jun},"\n";
 		} elsif ($argdo eq 'x') {
-		    print "Dummy date '$orig'.\n" if $verbose;
+                    if ($verbose) {
+                        print "Dummy date '$orig'.\n" if $verbose;
+                    }
 		    $date1->{date}  = 32;
 		    $date1->{month} = 13;
 		}
 	    }
 	    my $edate;
 	    if ($date2) {
-		if ($callbacks->{make_date_interval}) {
+		if ($callbacks &&
+                    $callbacks->{make_date_interval}) {
 		    $edate = &{$callbacks->{make_date_interval}} ($date1, $date2);
-		} else {
+		}
+                else {
 		    $edate = make_date_interval ($date1, $date2);
 		}
-	    } else {
-		if ($callbacks->{make_date}) {
+	    }
+            else {
+		if ($callbacks &&
+                    $callbacks->{make_date}) {
 		    $edate = &{$callbacks->{make_date}}($date1);
-		} else {
+		}
+                else {
 		    $edate = make_date ($date1);
 		}
 	    }
-	    print "-> '$edate'\n" if $verbose;
+            if ($verbose) {
+                print "-> '$edate'\n" if $verbose;
+            }
 	    $text =~ s/\Q$orig\E/$edate/g;
 	    if ($callbacks->{replace}) {
 		&{$callbacks->{replace}}($callbacks->{data}, $orig, $edate);
@@ -766,37 +537,319 @@ sub subsjdate
     return $text;
 }
 
-=head1 Author
+1;
 
-Ben Bullock, benkasminbullock@gmail.com
+__END__
+
+=encoding UTF-8
+
+=head1 NAME
+
+Lingua::JA::FindDates - scan text to find dates in a Japanese format
+
+=head1 SYNOPSIS
+
+  # Find and replace Japanese dates:
+
+  use Lingua::JA::FindDates 'subsjdate';
+
+  # Given a string, find and substitute all the Japanese dates in it.
+
+  my $dates = '昭和４１年三月１６日';
+  print subsjdate ($dates);
+
+  # prints "March 16, 1966"
+
+  # Find and substitute Japanese dates within a string:
+
+  my $dates = 'blah blah blah 三月１６日';
+  print subsjdate ($dates);
+
+  # prints "blah blah blah March 16"
+
+  # subsjdate can also call back a user-supplied routine each time a
+  # date is found:
+
+  sub replace_callback
+  {
+    my ($data, $before, $after) = @_;
+    print "'$before' was replaced by '$after'.\n";
+  }
+  my $dates = '三月１６日';
+  my $data = 'xyz'; # something to send to replace_callback
+  subsjdate ($dates, {replace => \&replace_callback, data => $data});
+
+  # prints "'三月１６日' was replaced by 'March 16'."
+
+  # A routine can be used to format the date any way, letting C<subsjdate>
+  # print it:
+
+  sub my_date
+  {
+    my ($date) = @_;
+    return join '/', $date->{month}."/".$date->{date};
+  }
+  my $dates = '三月１６日';
+  print subsjdate ($dates, {make_date => \&my_date});
+
+  # This prints "3/16"
+
+=head1 DESCRIPTION
+
+This module offers pattern matching of dates in the Japanese
+language. Its main routine, L</subsjdate> scans a text and finds
+things which appear to be Japanese dates.
+
+The module recognizes the typical format of dates with the year first,
+followed by the month, then the day, such as 平成20年七月十日
+I<(Heisei nijūnen shichigatsu tōka)>. It also recognizes combinations
+such as years alone, years and months, a month and day without a year,
+fiscal years (年度, "nendo"), parts of the month, like 中旬 (chūjun,
+the middle of the month), and periods between two dates.
+
+It recognizes both the Japanese-style era-base year format, such as 
+"平成２４年" (Heisei) for the current era, and European-style Christian
+era year format, such as 2012年. It recognizes several forms of
+numerals, including the ordinary ASCII numerals, 1, 2, 3; the "wide"
+or "double width" numerals sometimes used in Japan, １, ２, ３; and
+the kanji-based numeral system, 一, 二, 三. It recognizes some special
+date formats such as 元年 for the first year of an era. It recognizes
+era names identified by their initial letters, such as S41年 for Shōwa
+41 (1966). It recognizes dates regardless of any spacing which might
+be inserted between individual Japanese characters, such as 
+"平 成 二 十 年 八 月".
+
+The input text must be marked as Unicode, in other words character
+data, not byte data.
+
+The module has been tested on several hundred of documents, and it
+should cope with all common Japanese dates. If you find that it cannot
+identify some kind of date within Japanese text, please report that as
+a bug.
+
+If you would like to see more examples of how this module works, look
+at the testing code in C<t/Lingua-JA-FindDates.t>.
+
+=head1 FUNCTIONS
+
+=head2 subsjdate
+
+   my $translation = subsjdate ($text);
+
+Translate Japanese dates into American dates. The first argument to
+C<subsjdate> is a string like "平成２０年７月３日（木）". The routine
+looks through the string to see if there is anything which appears to
+be a Japanese date. If it finds one, it calls L</make_date> to make
+the equivalent date in English (American-style), and then substitutes
+it into C<$text>, as if performing the following type of operation:
+
+   $text =~ s/平成２０年７月３日（木）/Thursday, July 3, 2008/g;
+
+Users can supply a different date-making function using the second
+argument. The second argument is a hash reference which may have the
+following members:
+
+=over
+
+=item replace
+
+    subsjdate ($text, {replace => \&my_replace, data => $my_data});
+    # Now "my_replace" is called as
+    # my_replace ($my_data, $before, $after);
+
+If there is a replace value in the callbacks, subsjdate calls it as a
+subroutine with the data in C<<$callbacks->{data}>> and the before and
+after string, in other words the matched date and the string with
+which it is to be replaced.
+
+=item data
+
+Any data you want to pass to L</replace>, above.
+
+=item make_date
+
+    subsjdate ($text, {make_date => \& mymakedate});
+
+This is a replacement for the default L</make_date> function. The
+default function turns the Japanese dates into American-style dates,
+so, for example, "平成10年11月12日" is turned into "November 12,
+1998". If you don't need to replace the default (if you want
+American-style dates), you can leave this blank. If, for example, you
+want dates in the form "Th 2008/7/3", you could write a routine like
+the following:
+
+   sub mymakedate
+   {
+       my ($date) = @_;
+       return qw{Bad Mo Tu We Th Fr Sa Su}[$date->{wday}].
+           $date->{year}.'/'.$date->{month}.'/'.$date->{date};
+   } 
+
+Your routine will be called in the same way as the default routine,
+L</make_date>. It is necessary to check for the hash values for the
+fields C<year>, C<month>, C<date>, and C<wday> being zero, since
+L</subsjdate> matches "month/day" and "year/month" only dates.
+
+=item make_date_interval
+
+This is a replacement for the L<make_date_interval> function. 
+
+  subsjdate ($text, {make_date_interval => \&mymakedateinterval});
+
+Your routine will be called in the same way as the default routine,
+L</make_date_interval>. Its arguments are two dates.
+
+=back
+
+=head1 DEFAULT CALLBACKS
+
+This section describes the default callback routines.
+
+=head2 make_date
+
+   # Monday 19th March 2012.
+   make_date ({
+       year => 2012,
+       month => 3,
+       date => 19,
+       wday => 1,
+   })
+
+C<make_date> is the default date-string-making routine. It turns the
+date information supplied to it into a string representing the
+date. C<make_date> is not exported.
+
+L<subsjdate>, given a date like 平成２０年７月３日（木） (Heisei year
+20, month 7, day 3, in other words "Thursday the third of July,
+2008"), passes C<make_date> a hash reference with values (year =>2008,
+month => 7, date => 3, wday => 4) for the year, month, date and day of
+the week. C<make_date> returns a string, 'Thursday, July 3, 2008'. If
+some fields of the date aren't defined, for example in the case of a
+date like ７月３日 (3rd July), the hash values for the keys of the
+unknown parts of the date, such as year or weekday, will be undefined.
+
+To replace the default routine C<make_date> with a different format,
+supply a C<make_date> callback to L<subsjdate>:
+
+  sub my_date
+  {
+    my ($date) = @_;
+    return join '/', $date->{month}."/".$date->{date};
+  }
+  my $dates = '三月１６日';
+  print subsjdate ($dates, {make_date => \&my_date});
+
+This prints
+
+  3/16
+
+=head2 make_date_interval
+
+   make_date_interval (
+   {
+   # 19 February 2010
+       year => 2010,
+       month => 2,
+       date => 19,
+   },
+   # Monday 19th March 2012.
+   {
+       year => 2012,
+       month => 3,
+       date => 19,
+       wday => 1,
+   },);
+
+This function is called when an interval of two dates, such as 平成３年
+７月２日〜９日, is detected. It makes a string to represent that
+interval in English. It takes two arguments, hash references to the
+first and second date. The hash references are in the same format as
+L<make_date>.
+
+This function is not exported. It is the default used by
+C<subsjdate>. You can use another function instead of this default by
+supplying a value C<make_date_interval> as a callback in L<subsjdate>.
+
+=head1 BUGS
+
+=over
+
+=item １０月第４月曜日
+
+"１０月第４月曜日", which means "the fourth Monday of October", comes
+out as "October第April曜日".
+
+=item 今年６月
+
+The module does not handle things like 今年 (this year), 去年 (last
+year), or 来年 (next year).
+
+=item 末日
+
+The module does not handle "末日" (matsujitsu) "the last day" (of a month).
+
+=item 土日祝日
+
+The module does not handle "土日祝日" (weekends and holidays).
+
+=item 年末年始
+
+The module does not handle "年末年始" (the new year period).
+
+=item No sanity check of Japanese era dates
+
+It does not detect that dates like 昭和百年 (Showa 100, an impossible
+year, since Showa 63 (1988) was succeeded by Heisei 1 (1989)) are
+invalid.
+
+=item Only goes back to Meiji
+
+The date matching only goes back to the Meiji era. There is
+L<DateTime::Calendar::Japanese::Era> if you need to go back further.
+
+=item Doesn't find dates in order
+
+For those supplying their own callback routines, note that the dates
+returned won't be in the order that they are in the text, but in the
+order that they are found by the regular expressions, which means that
+in a string with two dates, the callbacks might be called for the
+second date before they are called for the first one. Basically the
+longer forms of dates are searched for before the shorter ones.
+
+=item UTF-8 version only
+
+This module only understands Japanese encoded in Perl's internal form
+(UTF-8).
+
+=item Trips a bug in Perl 5.10
+
+If you send subsjdate a string which is pure ASCII, you'll get a
+stream of warning messages about "uninitialized value". The error
+messages are wrong - this is actually a bug in Perl, reported as bug
+number 56902
+(L<http://rt.perl.org/rt3/Public/Bug/Display.html?id=56902>). But
+sending this routine a string which is pure ASCII doesn't make sense
+anyway, so don't worry too much about it.
+
+=item Doesn't do 元日 (I<ganjitsu>)
+
+This date (another way to write "1st January") is a little difficult,
+since the characters which make it up could also occur in other
+contexts, like 元日本軍 I<gennihongun>, "the former Japanese
+military". Correctly parsing it requires a linguistic analysis of the
+text, which this module isn't able to do.
+
+=back
 
 =cut
 
-# =head1 History
+=head1 EXPORTS
 
-# This routine started life as a Visual Basic for Applications (VBA)
-# script (a "Word Macro") to automatically convert Japanese dates in a
-# Microsoft Word document into their equivalent English versions. See
-# http://linuxtnt.wordpress.com/2008/04/16/visual-basic-date-translator-updated/
-# . Eventually, because I kept finding exceptions & I didn't know Visual
-# Basic well enough to code that efficiently, I decided to rewrite it
-# all in Perl, using L<Win32::OLE> to automate the operation of
-# Microsoft Word. (The Microsoft Word handlers are not included in this
-# module.)
+This module exports one function, L<subsjdate>, on request.
 
-# The basic idea is to ask Word to save a copy of the file as text via
-# OLE, then read the text file in to Perl, look for dates in the text
-# using L<subsjdate>, and then call back into Microsoft Word using the
-# L<replace> callback argument to L<subsjdate> to substitute the
-# Japanese dates with English ones.
+=cut
 
-=head1 Motivation
-
-The motivation for creating this module was as a form of assistance
-for translation of documents from Japanese into English, especially
-documents containing a large number of dates. 
-
-=head1 See also
+=head1 SEE ALSO
 
 These other modules might be more suitable for some purposes:
 
@@ -804,41 +857,38 @@ These other modules might be more suitable for some purposes:
 
 =item L<DateTime::Locale::JA>
 
-This does the minimal stuff to make a Japanese date. One of those
-modules which has been made for completeness rather than for
-usefulness, it doesn't represent Japanese language usages very well,
-failing to contain Japanese eras, kanji numbers, wide numbers, etc.
+Minimal selection of Japanese date functions. It's not complete enough
+to deal with the full range of dates in actual documents.
 
 =item L<DateTime::Format::Japanese>
 
 This parses Japanese dates. Unlike the present module it claims to
 also format them, so it can turn a L<DateTime> object into a Japanese
-date, and it also does times. However, the module seems to be broken -
-it doesn't install on any system I've tried.
+date, and it also does times. 
 
 =item L<Lingua::JA::Numbers>
 
-This module has a very full set of kanji / numeral convertors. It
-converts numbers including decimal points and numbers into the
-billions and trillions.
+Kanji / numeral convertors. It converts numbers including decimal
+points and numbers into the billions and trillions.
 
 =item L<DateTime::Calendar::Japanese::Era>
 
-This module contains a full set of Japanese eras.
+A full set of Japanese eras.
 
 =back
 
-=head1 COPYRIGHT AND LICENCE
+=head1 AUTHOR
 
-Copyright (C) 2008 Ben Kasmin Bullock.
-
-
-This module is distributed under the same terms as Perl itself, either
-Perl version 5.10.0 or, at your option, any later version of Perl 5
-you may have available.
+Ben Bullock, <bkb@cpan.org>
 
 =cut
 
-1;
+=head1 COPYRIGHT AND LICENCE
+
+Copyright (C) 2010-2012 Ben Bullock.
+
+You may use, copy, distribute, and modify this module under the same
+terms as the Perl programming language.
+
+=cut
 
-__END__
